﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
namespace MurdocTool.ClipConv
{
    class ClipConverter
    {
        static void Main(string[] args)
        {
            Options opt = Options.Analyze(args);
            ClipFile.Convert(opt);
        }
    }

    public enum ConvertMode
    {
        Convert , PathRewite
    }

    public class Options
    {
        /// <summary>元のClipファイルのPATH</summary>
        public string PathOriginalBin { get; set; }
        /// <summary>出力されるClipファイルのPATH</summary>
        public string PathOutputBin { get; set; }
        /// <summary>TSファイルのPATH</summary>
        public string PathTS { get; set; }
        /// <summary>GOPファイルのPATH</summary>
        public string PathGOP { get; set; }
        /// <summary>処理方法</summary>
        public ConvertMode Mode { get; set; }

        public static Options Analyze(string[] args)
        {
            Options aa = new Options();
            int i = 0;
            while (i < args.Length)
            {
                if (args[i][0] == '/')
                {
                    switch (char.ToUpper(args[i][1]))
                    {
                    case 'B':
                        aa.PathOriginalBin=args[++i];
                        break;
                    case 'G':
                         aa.PathGOP = args[++i];
                        break;
                    case 'O':
                        aa.PathOutputBin = args[++i];
                        break;
                    case 'T':
                        aa.PathTS = args[++i];
                        break;
                    case 'R':
                        aa.Mode = ConvertMode.PathRewite;
                        break;
                    }
                }
                i++;
            }
            return aa;
        }
 
    }

    /// <summary>
    /// MurdocCutterのClipFileを表すオブジェクト
    /// </summary>
    [System.Runtime.InteropServices.StructLayoutAttribute(System.Runtime.InteropServices.LayoutKind.Explicit , CharSet = CharSet.Ansi)] 
    public class ClipFile
    {
        [FieldOffset(0)]
        public ClipHeader Header;
        [FieldOffset(12)]
        public ClipItem[] Items;

        /// <summary>
        /// binファイルを読み込みこんでClipFileオブジェクトを作成します
        /// </summary>
        /// <param name="path">読み取るファイルパス</param>
        /// <returns></returns>
        public static ClipFile Load(string path)
        {
            using (System.IO.FileStream fs = new System.IO.FileStream(path , System.IO.FileMode.Open , System.IO.FileAccess.Read))
            {
                int size;
                size = Marshal.SizeOf(typeof(ClipHeader));
                byte[] bs;
                bs = new byte[size];
                fs.Read(bs , 0 , bs.Length);

                ClipHeader ch;
                {
                    GCHandle handle = GCHandle.Alloc(bs , GCHandleType.Pinned);

                    ch = (ClipHeader)Marshal.PtrToStructure(handle.AddrOfPinnedObject() , typeof(ClipHeader));
                    handle.Free();
                }

                ClipItem[] items = new ClipItem[ch.ListCount];
                size = Marshal.SizeOf(typeof(ClipItem));
                bs = new byte[size];
                for (int i = 0; i < ch.ListCount; i++)
                {
                    fs.Read(bs , 0 , bs.Length);
                    GCHandle handle = GCHandle.Alloc(bs , GCHandleType.Pinned);
                    items[i] = (ClipItem)Marshal.PtrToStructure(handle.AddrOfPinnedObject() , typeof(ClipItem));
                    handle.Free();
                }
                ClipFile cf = new ClipFile();
                cf.Header = ch;
                cf.Items = items;
                return cf;
            }
        }

        /// <summary>
        /// binファイルを指定して書き出します。
        /// </summary>
        /// <param name="path">書き出すファイルパス</param>
        public void Save(string path)
        {
            int size = Marshal.SizeOf(typeof(ClipHeader)) + Marshal.SizeOf(typeof(ClipItem)) * Items.Length;
            byte[] bs = new byte[size];
            GCHandle handle = GCHandle.Alloc(bs , GCHandleType.Pinned);
            Marshal.StructureToPtr(this.Header , handle.AddrOfPinnedObject() , true);
            IntPtr ip = handle.AddrOfPinnedObject();
            long p = ip.ToInt64() + Marshal.SizeOf(typeof(ClipHeader));
            for (int i = 0; i < Items.Length; i++)
            {
                Marshal.StructureToPtr(this.Items[i] , new IntPtr(p) , true);
                p += Marshal.SizeOf(typeof(ClipItem));
            }
            handle.Free();
            using (System.IO.FileStream fs = new System.IO.FileStream(path , System.IO.FileMode.Create , System.IO.FileAccess.Write))
            {
                fs.Write(bs , 0 , bs.Length);
            }
        }

        //public void CreateGOPFile(string originalGOPFile , string outputPath)
        //{
        //    List<ulong> ptss = new List<ulong>();
        //    List<string> sptss = new List<string>();
        //    foreach (ClipItem item in Items)
        //    {
        //        if (!ptss.Contains(item.PTSStart))
        //        {
        //            ptss.Add(item.PTSStart);
        //            sptss.Add(item.PTSStart.ToString());
        //        }
        //        if (!ptss.Contains(item.PTSEnd))
        //        {
        //            ptss.Add(item.PTSEnd);
        //            sptss.Add(item.PTSEnd.ToString());
        //        }
        //    }

        //    using (System.IO.StreamReader sr = new System.IO.StreamReader(originalGOPFile))
        //    {
        //        string line;
        //        line = sr.ReadLine();
        //        if (line.StartsWith("TS_CUT"))
        //        {
        //            using (System.IO.StreamWriter sw = new System.IO.StreamWriter(outputPath))
        //            {
        //                sw.WriteLine(line);

        //                string[] values;
        //                ulong ulSum=0;
        //                while (sr.Peek() != -1)
        //                {
        //                    line = sr.ReadLine();
        //                    values = line.Split(',');
        //                    if (values.Length >= 5)
        //                    {
        //                        ulong ul;
        //                        if (!ulong.TryParse(values[4] , out ul))
        //                        {
        //                            throw new System.IO.InvalidDataException();
        //                        }
        //                        ulSum += ul;
        //                        int foundIndex;
        //                        //foundIndex = sptss.IndexOf(values[3].Trim());
        //                        foundIndex = ptss.IndexOf(ulong.Parse(values[3]));
        //                        if (foundIndex >= 0)
        //                        {
        //                            sptss.RemoveAt(foundIndex);
        //                            ptss.RemoveAt(foundIndex);
        //                            values[4] = ulSum.ToString();
        //                            ulSum = 0;
        //                            sw.WriteLine(string.Join("," , values));
        //                        }
        //                    }
        //                    else
        //                    {
        //                        sw.WriteLine(line);
        //                    }
        //                }
        //            }
        //        }
        //    }
        //    if (sptss.Count > 0)
        //    {
        //        throw new System.IO.InvalidDataException("元のGOPファイルが必要なPTSを含んでいません");
        //    }
        //}

        private class PTSCheck
        {
            public bool HasStart;
            public bool HasEnd;
        }

        /// <summary>
        /// Clipファイルに記述されているTSファイルのPATHを書き換えて保存する
        /// </summary>
        /// <param name="tsFilePath">新しいTSファイルのPATH</param>
        /// <param name="newClipFilePath">新しいClipファイルのPATH</param>
        /// <param name="originalClipFilePath">元のClipファイルのPATH</param>
        public static void RewriteTSFilePath(string tsFilePath , string newClipFilePath , string originalClipFilePath)
        {
            ClipFile clip = ClipFile.Load(originalClipFilePath);
            clip.RewriteTSFilePath(tsFilePath);
            clip.Save(newClipFilePath);
        }

        /// <summary>
        /// Clipファイルに記述されているTSファイルのPATHを書き換える
        /// </summary>
        /// <param name="tsFilePath">新しいTSファイルのPATH</param>
        private void RewriteTSFilePath(string tsFilePath)
        {
            byte[] bsTS = System.Text.Encoding.Default.GetBytes(tsFilePath);
            if (bsTS.Length >= 500)
            {
                throw new ArgumentOutOfRangeException("新しいファイルのパスが長すぎます");
            }

            string pathCheck = string.Empty;
            foreach (ClipItem item in this.Items)
            {
                if (pathCheck == string.Empty)
                {
                    pathCheck = item.FilePath;
                }
                else if (pathCheck != item.FilePath)
                {
                    throw new ApplicationException("複数のTSファイルを含むClipFileはサポートしていません");
                }
            }

            foreach (ClipItem item in this.Items)
            {
                item.FilePath = tsFilePath;
            }
        }


        /// <summary>
        /// このクリップファイルを別のGOPファイルに合うように変換します
        /// </summary>
        /// <param name="gopFilePath">新しいTSファイルのGOPファイルのPATH</param>
        /// <param name="newClipFilePath">このファイルの書き出しPATH</param>
        public void Convert(string gopFilePath , string newClipFilePath)
        {
            Dictionary<ClipItem , PTSCheck> dic = new Dictionary<ClipItem , PTSCheck>();
            foreach (ClipItem item in Items)
            {
                dic.Add(item , new PTSCheck());
            }

            using (System.IO.StreamReader sr = new System.IO.StreamReader(gopFilePath))
            {
                string line;
                line = sr.ReadLine();
                if (!line.StartsWith("TS_CUT"))
                {
                    throw new System.IO.InvalidDataException("GOPファイルを認識できませんでした");
                }
                else
                {
                    string[] values;
                    ulong ulPacketIndexSum = 0;
                    uint index= 0;;
                    while (sr.Peek() != -1)
                    {
                        line = sr.ReadLine();
                        values = line.Split(',');
                        if (values.Length >= 5)
                        {
                            ulong ulPts;
                            ulong ulPacketIndex;

                            if (!ulong.TryParse(values[3],out ulPts) || !ulong.TryParse(values[4] , out ulPacketIndex))
                            {
                                throw new System.IO.InvalidDataException("GOPファイルのデータを認識できませんでした");
                            }
                            ulPacketIndexSum += ulPacketIndex;


                            foreach (ClipItem item in dic.Keys)
                            {
                                if (item.PTSStart == ulPts)
                                {
                                    if (!dic[item].HasStart )
                                    {
                                        item.StartIndex = index;
                                        item.StartPacketIndex = ulPacketIndexSum;
                                        dic[item].HasStart  = true;
                                    }
                                }
                                if (item.PTSEnd == ulPts)
                                {
                                    if (!dic[item].HasEnd )
                                    {
                                        item.EndIndex = index;
                                        item.EndPacketIndex = ulPacketIndexSum;
                                        dic[item].HasEnd  = true;
                                    }
                                }
                            }
                            index++;
                        }
                        else if (values.Length==2 && line.StartsWith("Last"))
                        {
                            foreach(ClipItem item in dic.Keys)
                            {
                                item.LastPacketIndex= ulong.Parse(values[1]);
                            }
                        }
                    }

                    foreach (PTSCheck chk in dic.Values)
                    {
                        if (!chk.HasEnd  || !chk.HasStart)
                        {
                            throw new System.IO.InvalidDataException("新しいGOPファイルには必要なPTSが含まれていませんでした");
                        }
                    }
                }
            }
            Save(newClipFilePath);
        }

        /// <summary>
        /// このクリップファイルを別のGOPファイルに合うように変換します
        /// </summary>
        /// <param name="tsFilePath">新しいTSファイルのPATH</param>
        /// <param name="gopFilePath">新しいTSファイルのGOPファイルのPATH</param>
        /// <param name="newClipFilePath">このファイルの書き出しPATH</param>
        /// <param name="originalClipFilePath">元のClipファイルのPATH</param>
        public static void Convert(string tsFilePath , string gopFilePath , string newClipFilePath,string originalClipFilePath )
        {
            ClipFile clip = ClipFile.Load(originalClipFilePath);
            clip.RewriteTSFilePath(tsFilePath);
            clip.Convert(gopFilePath , newClipFilePath);
            clip.Save(newClipFilePath);
        }

        /// <summary>
        /// オプションを指定して変換する
        /// </summary>
        /// <param name="opt"></param>
        public static void Convert(Options opt)
        {
            if (opt.Mode == ConvertMode.PathRewite)
            {//TSファイルのPATHのみ書き換え
                RewriteTSFilePath(opt.PathTS , opt.PathOutputBin , opt.PathOriginalBin);
            }
            else
            {
                Convert(opt.PathTS , opt.PathGOP , opt.PathOutputBin , opt.PathOriginalBin);
            }
        }
    }

    [System.Runtime.InteropServices.StructLayoutAttribute(System.Runtime.InteropServices.LayoutKind.Explicit , CharSet = CharSet.Ansi , Pack = 1)]
    public class ClipHeader
    {
        [FieldOffset(0)]
        [MarshalAs(UnmanagedType.ByValArray , ArraySubType = UnmanagedType.U1 , SizeConst = 8)]
        public byte[] Magic;
        [FieldOffset(8)]
        public ushort Unknown;
        [FieldOffset(10)]
        public ushort ListCount;
    }


    [System.Runtime.InteropServices.StructLayoutAttribute(System.Runtime.InteropServices.LayoutKind.Explicit , CharSet = CharSet.Ansi , Pack = 1)] 
    public class ClipItem
    {
        [FieldOffset(0) , MarshalAs(UnmanagedType.ByValTStr , SizeConst = 250)]
        public string FilePath;
        [FieldOffset(500)]
        public uint StartIndex;
        [FieldOffset(504)]
        public uint EndIndex;
        [FieldOffset(508)]
        public ulong StartPacketIndex;
        [FieldOffset(516)]
        public ulong EndPacketIndex;
        [FieldOffset(524)]
        public ulong LastPacketIndex;
        [FieldOffset(532)]
        public ulong PTSStart;
        [FieldOffset(540)]
        public ulong PTSEnd;
        [FieldOffset(548)]
        public uint PacketSize;
        [FieldOffset(552)]
        public uint Unknown1;
        [FieldOffset(556)]
        [MarshalAs(UnmanagedType.ByValArray , ArraySubType = UnmanagedType.U1 , SizeConst = 6)]
        public byte[] Unknown2;
    }
}
