﻿using System;
using System.Windows.Forms;
//ClipConvとMurdocBatchを参照すること

namespace MurdocTool
{
    public partial class ToolForm : Form
    {
        private static  string iniPath;

        /// <summary>
        /// アプリケーションのメイン エントリ ポイントです。
        /// </summary>
        [STAThread]
        static void Main()
        {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);

            iniPath = System.IO.Path.ChangeExtension(typeof(ToolForm).Assembly.Location , ".ini");

            var options = MurdocTool.Options.Analyze(System.Environment.GetCommandLineArgs());

            if (!System.IO.File.Exists(options.MurdocFiles.ExePath))
            {
                options.MurdocFiles.SetPath(API.GetPrivateProfileString(SETTING,"ExePath",MurdocTool.MurdocFiles.DEFAULT_EXE_FILENAME,iniPath));
            }

            if (!System.IO.File.Exists(options.MurdocFiles.ExePath))
            {
                using (System.Windows.Forms.OpenFileDialog dlg = new OpenFileDialog())
                {
                    dlg.AddExtension = true;
                    dlg.CheckFileExists = true;
                    dlg.CheckPathExists = true;
                    dlg.DefaultExt = ".exe";
                    dlg.FileName = MurdocTool.MurdocFiles.DEFAULT_EXE_FILENAME;
                    dlg.Filter = "MurdocCutter|*.exe";
                    dlg.FilterIndex = 0;
                    dlg.Title = "MurdocCut.exeを選択";
                    dlg.InitialDirectory = System.Environment.GetFolderPath(System.Environment.SpecialFolder.ProgramFiles);
                    if (dlg.ShowDialog() == DialogResult.OK )
                    {
                        options.MurdocFiles.SetPath(dlg.FileName);
                    }
                        
                }
            }

            var filestatus = options.MurdocFiles.CheckFiles();
            if (filestatus != MurdocFiles.FileStatus.Success)
            {
                System.Windows.Forms.MessageBox.Show(MurdocFiles.GetStatusString(filestatus));
            }
            else
            {
                System.Diagnostics.ProcessStartInfo pinfo
                    = new System.Diagnostics.ProcessStartInfo(options.MurdocFiles.ExePath);

                System.Diagnostics.Process process
                    = new System.Diagnostics.Process();

                process.StartInfo = pinfo;
                process.StartInfo.UseShellExecute = false;
                process.Start();
                process.WaitForInputIdle();
                ToolForm frm = new ToolForm();

                frm.Show(process);
                frm.options = options;

                Application.Run(frm);

                API.WritePrivateProfileString("SETTING" , "ExePath" , options.MurdocFiles.ExePath , iniPath);

                //Properties.Settings.Default.Save();
            }
        }

        public ToolForm()
        {
            InitializeComponent();

            System.Drawing.Point startpoint
                = new System.Drawing.Point(
                    GetPrivateProfileInteger(SETTING , "Left" , this.Left) ,
                    GetPrivateProfileInteger(SETTING , "Top" , this.Top));
            System.Windows.Forms.Screen screen = System.Windows.Forms.Screen.FromPoint(startpoint);
            if (!screen.WorkingArea.Contains(startpoint))
            {
                startpoint = screen.WorkingArea.Location;
            }
            this.StartPosition = FormStartPosition.Manual;
            this.Location = startpoint;
            this.chAutoClearList.Checked = "0" != API.GetPrivateProfileString(SETTING , "AutoListClear" , string.Empty , iniPath);
            this.chAutoFileRename .Checked = "0" != API.GetPrivateProfileString(SETTING , "AutoRename" , string.Empty , iniPath);
            //this.txbMark.Text = API.GetPrivateProfileString(SETTING , "Mark" , string.Empty , iniPath);

            string[] marks = API.GetPrivateProfileString(SETTING , "Mark" , string.Empty , iniPath).Split(new char[]{'|'}, StringSplitOptions.RemoveEmptyEntries);
            this.cbxMarks.Items.AddRange(marks);
            this.raMarkAfter.Checked = "0"!=API.GetPrivateProfileString(SETTING , "IsMarkAfter" , "0" , iniPath);

            string gop = API.GetPrivateProfileString(SETTING , "GOPQuestion" , opGOPNo.Text , iniPath);
            foreach (Control ctl in grpGOP.Controls)
            {
                RadioButton rb = ctl as RadioButton;
                if (rb != null && rb.Text== gop)
                {
                    rb.Checked=true;
                }
            }

            int i = 0;
            int.TryParse(API.GetPrivateProfileString(SETTING , "MinimumMinute" , "0" , iniPath),out i);
            numMinimumMinute.Value = i;
        }

        public void Show(System.Diagnostics.Process process)
        {
            process.EnableRaisingEvents = true;
            process.Exited += new EventHandler(process_Exited);
            if (options != null)
            {
                mw = new MurdocWindow(process , options.ClippingButtonText);
            }
            else
            {
                mw = new MurdocWindow(process,MurdocTool.Options.DefaultClippingButtonText );
            }

            this.btnLoad.Enabled = System.IO.Directory.Exists(mw.MurdocFiles.GOPFilePath);

            this.Show(mw);
        }

        protected override void OnClosing(System.ComponentModel.CancelEventArgs e)
        {
            e.Cancel = !this.mw.Process.HasExited;
            base.OnClosing(e);

            if (!e.Cancel)
            {
                API.WritePrivateProfileString(SETTING , "Top" , this.Top.ToString() , iniPath);
                API.WritePrivateProfileString(SETTING , "Left" , this.Left.ToString() , iniPath);
                //API.WritePrivateProfileString(SETTING , "Mark" , this.txbMark.Text , iniPath);

                System.Collections.Generic.List<string> marks = new System.Collections.Generic.List<string>();
                foreach (string s in cbxMarks.Items)
                {
                    marks.Add(s);
                }
                API.WritePrivateProfileString(SETTING , "Mark" , string.Join("|" , marks.ToArray()), iniPath);
                API.WritePrivateProfileString(SETTING , "IsMarkAfter" , raMarkAfter.Checked ? "1" : "0" , iniPath);


                API.WritePrivateProfileString(SETTING , "AutoListClear" , chAutoClearList.Checked ? "1" : "0" , iniPath);
                API.WritePrivateProfileString(SETTING , "AutoRename" , chAutoFileRename.Checked ? "1" : "0" , iniPath);
                foreach(Control ctl in grpGOP.Controls)
                {
                    RadioButton rb = ctl as RadioButton;
                    if(rb != null && rb.Checked)
                    {
                        API.WritePrivateProfileString(SETTING , "GOPQuestion" , rb.Text,iniPath );
                    }
                }
                API.WritePrivateProfileString(SETTING , "MinimumMinute" , numMinimumMinute.Value.ToString() , iniPath);
            }
        }

        private const string SETTING ="Setting";

        private int GetPrivateProfileInteger(string section , string key , int defaultvalue)
        {
            int i;
            if (int.TryParse(API.GetPrivateProfileString(section , key , defaultvalue.ToString() , iniPath) , out i))
            {
                return i;
            }
            else
            {
                return defaultvalue;
            }
        }

        private MurdocTool.Options options=null;
        private MurdocTool.MurdocWindow mw;
        private bool IsSaved
        {
            get
            {
                return _IsSaved;
            }
            set
            {
                if(value)
                {
                    oldListDatas = mw.GetListData();
                }
                _IsSaved=value;
            }
        }
        private bool _IsSaved = false;

        private System.Collections.Generic.List<ListData> oldListDatas = new System.Collections.Generic.List<ListData>();

        void process_Exited(object sender , EventArgs e)
        {
            MethodInvoker del= delegate(){this.Close();};
            this.Invoke(del);
        }

        private string MakeBinPath()
        {
            string path = mw.OutputPath;
            if (chMark.Checked && !string.IsNullOrEmpty(cbxMarks.Text ))
            {
                string dir = System.IO.Path.GetDirectoryName(path);
                string name;
                if (raMarkBefore.Checked )
                {
                    name = cbxMarks.Text + System.IO.Path.GetFileName(path);
                }
                else
                {
                    name = System.IO.Path.GetFileNameWithoutExtension(path) + cbxMarks.Text + System.IO.Path.GetExtension(path);
                }
                path= System.IO.Path.Combine(dir , name);
            }
            path = System.IO.Path.ChangeExtension(path , ".bin");
            return path;
        }

        private void btnSaveClip_Click(object sender , EventArgs e)
        {
            Save();
        }

        private void Save()
        {
            string binPath = MakeBinPath();
            switch (mw.SaveClip(binPath))
            {
            case MurdocWindow.SaveResult.Success:
                TreeNode nodeBin = new TreeNode(binPath);

                TimeSpan totalTime = TimeSpan.Zero;

                foreach (ListData ld in mw.GetListData())
                {
                    TreeNode listNode;
                    listNode = new TreeNode();
                    listNode.Text = ld.OriginalString;
                    //listNode.ToolTipText = ld.Length.ToString();
                    listNode.Tag = ld;
                    nodeBin.Nodes.Add(listNode);

                    totalTime += ld.Length;
                }
                this.treeView1.Nodes.Add(nodeBin);


                if (chMark.Checked && !string.IsNullOrEmpty(this.cbxMarks.Text ))
                {
                    bool contain = false;
                    string text = cbxMarks.Text;
                    foreach (string s in cbxMarks.Items)
                    {
                        if (s == text)
                        {
                            contain = true;
                        }
                    }
                    if (!contain)
                    {
                        cbxMarks.Items.Add(text);
                    }
                }

                chMark.Checked = false;
                IsSaved = true;

                if (totalTime.TotalSeconds < (double)(numMinimumMinute.Value * 60))
                {
                    System.Windows.Forms.MessageBox.Show(this , "切り出し時間が短いかもしれません");
                }

                break;
            case MurdocWindow.SaveResult.NoList:
                break;
            case MurdocWindow.SaveResult.NoSaveDialog:
                System.Windows.Forms.MessageBox.Show(this , "セーブダイアログが見つかりませんでした");
                break;
            case MurdocWindow.SaveResult.NoSaveFile:
                System.Windows.Forms.MessageBox.Show(this , "binファイルの書き込みを検出できませんでした。");
                break;
            }
        }

        private void timer1_Tick(object sender , EventArgs e)
        {
            timer1.Stop();
            try
            {
                if(!opGOPNone.Checked)
                {
                    var dialogs = mw.FindDialogs();
                    if (dialogs.Count > 0)
                    {
                        foreach (IntPtr hDialog in dialogs.Keys)
                        {
                            if (dialogs[hDialog] == "question" && API.IsWindowVisible(hDialog))
                            {
                                IntPtr hButton = IntPtr.Zero;
                                if (opGOPYes.Checked)
                                {
                                    hButton = API.GetDlgItem(hDialog , 0x445);
                                }
                                else if (opGOPNo.Checked)
                                {
                                    hButton = API.GetDlgItem(hDialog , 0x446);
                                }
                                else if (opGOPCancel.Checked)
                                {
                                    hButton = API.GetDlgItem(hDialog , 0x449);
                                }

                                if (hButton != IntPtr.Zero)
                                {
                                    API.SendMessage(hButton , (uint)API.WM.WM_LBUTTONDOWN , IntPtr.Zero , IntPtr.Zero);
                                    API.SendMessage(hButton , (uint)API.WM.WM_LBUTTONUP , IntPtr.Zero , IntPtr.Zero);//SendMessageだとエラーのときに止まる

                                }
                            }
                        }
                    }

                }
                var list = mw.GetListData();
                if (list.Count>=2)
                {
                    for (int i = 0; i < list.Count - 1; i++)
                    {
                        if (list[i].File.Equals(list[i + 1].File , StringComparison.OrdinalIgnoreCase))
                        {
                            TimeSpan span;
                            span = list[i + 1] - list[i];
                            if (span.Ticks < 0)
                            {
                                ShowMessageBox("時間が逆転しています");
                                break;
                            }
                            if (span.TotalSeconds >= ListData.CUT_SPAN_LIMIT)
                            {
                                ShowMessageBox("カット時間が長いです");
                                break;
                            }
                        }
                    }
                }
                if (IsSaved)
                {
                    if (oldListDatas.Count != list.Count)
                    {
                        IsSaved = false;
                    }
                    else
                    {
                        for (int i = 0; i < list.Count; i++)
                        {
                            if (!oldListDatas[i].Equals(list[i]))
                            {
                                IsSaved = false;
                                break;
                            }
                        }
                    }
                }


                if (!mw.HasForcusNewPath)
                {
                    string org = mw.OriginalPath;
                    string ts = mw.OutputPath;
                    if (org.Trim().Length > 0)
                    {
                        if (!System.IO.Path.GetFileNameWithoutExtension(org)
                            .Equals(System.IO.Path.GetFileNameWithoutExtension(ts)))
                        {

                            if (chAutoClearList.Checked)
                            {
                                if (!IsSaved)
                                {
                                    if (mw.ListCount > 0)
                                    {
                                        switch (System.Windows.Forms.MessageBox.Show(
                                            this ,
                                            "保存されていません。\r\n保存しますか?"
                                            , string.Empty
                                            , MessageBoxButtons.YesNo))
                                        {
                                        case DialogResult.Yes:
                                            Save();
                                            if (IsSaved)// 保存に成功してたら
                                            {
                                                mw.ClickClearButton();
                                            }
                                            break;
                                        case DialogResult.No:
                                            mw.ClickClearButton();
                                            IsSaved = true;
                                            oldListDatas = new System.Collections.Generic.List<ListData>();
                                            break;
                                        }
                                    }
                                    else
                                    {
                                        IsSaved = true;
                                    }
                                }
                                else
                                {
                                    mw.ClickClearButton();
                                    IsSaved = true;
                                    oldListDatas = new System.Collections.Generic.List<ListData>();
                                }
                            }

                            if (chAutoFileRename.Checked)
                            {
                                ts = System.IO.Path.Combine(
                                    System.IO.Path.GetDirectoryName(ts) ,
                                    System.IO.Path.GetFileNameWithoutExtension(org)
                                    + System.IO.Path.GetExtension(ts));
                                mw.OutputPath = ts;
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                ShowMessageBox(ex.Message);
                this.Close();
                return;
            }
            timer1.Start();
        }

         private void ShowMessageBox(string msg)
        {
            API.MessageBox(this.Handle , msg , this.Text
                , API.MessageBoxOptions.Ok
                | API.MessageBoxOptions.SetForeground
                | API.MessageBoxOptions.ApplicationModal
                | API.MessageBoxOptions.Topmost
                | API.MessageBoxOptions.IconExclamation);
        }

         private void btnSeek_Click(object sender , EventArgs e)
         {
             try
             {
                 mw.SeekPTSByList(sender == btnSeekStart);
             }
             catch (Exception ex)
             {
                 ShowMessageBox(ex.Message);
             }
         }

         private void contextClear_Click(object sender , EventArgs e)
         {
             this.treeView1.Nodes.Clear();
         }

         private void btnLoad_Click(object sender , EventArgs e)
         {
             OpenFileDialog dlg = new OpenFileDialog();
             dlg.Filter = "ClipList|*.bin";
             if (dlg.ShowDialog() == DialogResult.OK)
             {
                 string ts;
                 string tsFileName;
                 string gop;
                 
                 string bin;
                 string newbin;
                 ts=mw.OriginalPath;
                 tsFileName=System.IO.Path.GetFileName(ts);
                 gop=System.IO.Path.Combine(mw.MurdocFiles.GOPFilePath,tsFileName= ".txt");
                 bin = dlg.FileName;
                 newbin=System.IO.Path.Combine(mw.OutputPath ,System.IO.Path.ChangeExtension(tsFileName,"bin"));
                 try
                 {
                     ClipConv.ClipFile.Convert(ts , gop , newbin , bin);
                 }
                 catch(Exception ex)
                 {
                     MessageBox.Show(ex.Message);
                 }
             }
         }




    }
}
